import itertools
import sys
import os
import urllib

import util
import pool

FetchLogName = 'log/fetch.log'

# http://www.nhl.com/stats/rest/skaters?isAggregate=false&reportType=basic&isGame=false&reportName=faceoffs&sort=[{%22property%22:%22faceoffsWon%22,%22direction%22:%22DESC%22}]&factCayenneExp=gamesPlayed%3E=1&cayenneExp=gameTypeId=2%20and%20seasonId%3E=20172018%20and%20seasonId%3C=20172018%20and%20teamId=53
# http://www.nhl.com/stats/rest/goalies?isAggregate=false&reportType=goalie_basic&isGame=false&reportName=goaliesummary&sort=[{%22property%22:%22wins%22,%22direction%22:%22DESC%22}]&cayenneExp=gameTypeId=2%20and%20seasonId%3E=20172018%20and%20seasonId%3C=20172018

def FetchNHL( dirName ) :
    for team in util.ActiveTeamNames() :
        fName = '%s/nhl_%s.json' % (dirName, team[0])
        util.ExecCommand( "wget -O %s -a %s 'http://www.nhl.com/stats/rest/skaters/?isAggregate=false&reportType=basic&isGame=false&reportName=faceoffs&cayenneExp=seasonId=20192020%%20and%%20gameTypeId=2%%20and%%20teamId=%s'" % (fName, FetchLogName, team[7]) )
    util.ExecCommand( "wget -O %s/nhl_goalies.json -a %s 'http://www.nhl.com/stats/rest/goalies?isAggregate=false&reportType=goalie_basic&isGame=false&reportName=goaliesummary&cayenneExp=seasonId=20192020%%20and%%20gameTypeId=2%%20and%%20playerPositionCode=%%22G%%22'" % (dirName, FetchLogName) )

# http://www.nhl.com/stats/rest/skaters?isAggregate=false&reportType=basic&isGame=false&reportName=bios&sort=[{%22property%22:%22playerBirthDate%22,%22direction%22:%22DESC%22}]&factCayenneExp=gamesPlayed%3E=1&cayenneExp=gameTypeId=2%20and%20seasonId%3E=20172018%20and%20seasonId%3C=20172018%20and%20teamId=24
# http://www.nhl.com/stats/rest/goalies?isAggregate=false&reportType=goalie_basic&isGame=false&reportName=goaliebios&sort=[{%22property%22:%22playerBirthDate%22,%22direction%22:%22DESC%22}]&cayenneExp=gameTypeId=2%20and%20seasonId%3E=20172018%20and%20seasonId%3C=20172018

def FetchNHLBio( dirName ) :
    for team in util.ActiveTeamNames() :
        fName = '%s/bio_%s.json' % (dirName, team[0])
        util.ExecCommand( "wget -O %s -a %s 'http://www.nhl.com/stats/rest/skaters?isAggregate=false&reportType=basic&isGame=false&reportName=bios&cayenneExp=seasonId=20192020%%20and%%20gameTypeId=2%%20and%%20teamId=%s'" % (fName, FetchLogName, team[7]) )
    util.ExecCommand( "wget -O %s/bio_goalies.json -a %s 'http://www.nhl.com/stats/rest/goalies?isAggregate=false&reportType=goalie_basic&isGame=false&reportName=goaliebios&cayenneExp=seasonId=20192020%%20and%%20gameTypeId=2%%20and%%20playerPositionCode=%%22G%%22'" % (dirName, FetchLogName) )

# http://sports.yahoo.com/nhl/players/2102

def FetchYahooBio( linkList ) :
    dirName = 'fetch/yahoobio'
    util.Mkdir( dirName )
    count = 0
    for link in linkList :
        id = link.rsplit( '/', 1 )[1]
        fName = '%s/id%s.html' % (dirName, id)
        if not os.path.exists( fName ) :
            count += 1
            util.ExecCommand( 'wget -O %s -a %s %s' % (fName, FetchLogName, link), 5 )
            # make sure we don't annoy Yahoo
            if count >= 500 : break

# http://ca.sports.yahoo.com/nhl/teams/ana/stats/

def FetchYahoo( dirName ) :
    for entry in util.ActiveTeamNames() :
        abbrev = entry[0]
        fName = '%s/yahoo_%s.html' % (dirName, abbrev)
        util.ExecCommand( "wget -O %s -a %s 'http://ca.sports.yahoo.com/nhl/teams/%s/stats/'" % (fName, FetchLogName, abbrev), 5 )

# http://www.capfriendly.com/teams/jets/salary

def FetchCapFriendly( dirName ) :
    for entry in util.ActiveTeamNames() :
        fName = '%s/capfriendly_%s.html' % (dirName, entry[0])
        teamName = entry[2].lower().replace( ' ', '' )
        util.ExecCommand( "wget -O %s -a %s 'http://www.capfriendly.com/teams/%s/salary'" % (fName, FetchLogName, teamName), 5 )

# did this fetch manually Y2016-18.   Can semi-automate next year hopefully!
# https://statsapi.web.nhl.com/api/v1/schedule?startDate=2016-10-12&endDate=2017-4-9

def FetchSchedule( dirName ) :
    util.ExecCommand( "wget -O %s/schedule.json -a %s 'https://statsapi.web.nhl.com/api/v1/schedule?startDate=2019-10-01&endDate=2020-04-05'" % (dirName, FetchLogName) )

# http://www.nhl.com/stats/rest/skaters?isAggregate=false&reportType=basic&isGame=false&reportName=faceoffs&sort=[{%22property%22:%22faceoffsWon%22,%22direction%22:%22DESC%22}]&factCayenneExp=gamesPlayed%3E=1&cayenneExp=gameTypeId=3%20and%20seasonId%3E=20172018%20and%20seasonId%3C=20172018%20and%20teamId=53
# http://www.nhl.com/stats/rest/goalies?isAggregate=false&reportType=goalie_basic&isGame=false&reportName=goaliesummary&sort=[{%22property%22:%22wins%22,%22direction%22:%22DESC%22}]&cayenneExp=gameTypeId=3%20and%20seasonId%3E=20172018%20and%20seasonId%3C=20172018

def FetchPlayoffs( dirName ) :
    util.ExecCommand( "wget -O %s/skaters.json -a %s 'http://www.nhl.com/stats/rest/skaters/?isAggregate=false&reportType=basic&isGame=false&reportName=faceoffs&cayenneExp=seasonId=20182019%%20and%%20gameTypeId=3'" % (dirName, FetchLogName) )
    util.ExecCommand( "wget -O %s/goalies.json -a %s 'http://www.nhl.com/stats/rest/goalies?isAggregate=false&reportType=goalie_basic&isGame=false&reportName=goaliesummary&cayenneExp=seasonId=20182019%%20and%%20gameTypeId=3%%20and%%20playerPositionCode=%%22G%%22'" % (dirName, FetchLogName) )

# http://nhlnumbers.com/team-salaries/chicago-blackhawks-salary-cap/

def FetchNHLNumbers( dirName ) :
    for team in util.ActiveTeamNames() :
        name = team[1].replace(' ', '-').lower()
        name = name.replace('.', '')
        fName = '%s/nhlnumbers_%s.html' % (dirName, team[5])
        util.ExecCommand( "wget -O %s -a %s 'http://nhlnumbers.com/team-salaries/%d/%s-salary-cap'" % (fName, FetchLogName, pool.PoolYear, name) )

# http://capgeek.com/canucks/

def FetchCapGeek( dirName ) :
    for team in util.ActiveTeamNames() :
        team = team[2].lower().replace( ' ', '' )
        fName = '%s/capgeek_%s.html' % (dirName, team)
        util.ExecCommand( "wget -O %s -a %s 'http://capgeek.com/%s'" % (fName, FetchLogName, team) )

# http://sports.yahoo.com/nhl/injuries

def FetchInjuries( dirName ) :
    util.ExecCommand( "wget -O %s/injuries.html -a %s 'http://sports.yahoo.com/nhl/injuries'" % (dirName, FetchLogName) )


def InitFetch() :
    util.Mkdir( 'log' )
    util.ExecCommand( 'rm -f log/fetch.log' )
    util.ExecCommand( 'touch log/fetch.log' )


FetchFuncDict = {
    'nhl' : FetchNHL,
    'nhlbio' : FetchNHLBio,
    'yahoo' : FetchYahoo,
    'schedule' : FetchSchedule,
    'playoffs' : FetchPlayoffs,
    'nhlnumbers' : FetchNHLNumbers,
    'injuries' : FetchInjuries,
    'capgeek' : FetchCapGeek,
    'capfriendly' : FetchCapFriendly,
}

def FetchGroupedData( which ) :
    dirName = 'fetch/' + which
    util.Mkdir( dirName )
    FetchFuncDict[ which ]( dirName )
